#include "StdAfx.h"
#include "GPGraphics.h"

CGPGraphics::CGPGraphics(IDirect3DDevice9 *pDevice, CGPComms* pComms)
{
    m_pDevice = pDevice;
    m_pComms  = pComms;

    m_pBackgroundTexture = NULL;
	m_pPicture = NULL;
	m_pIMPicture = NULL;

    FontSetup();
}

CGPGraphics::~CGPGraphics(void)
{
    SAFERELEASE(m_pBackgroundTexture);
	SAFERELEASE(m_pPicture);
	SAFERELEASE(m_pIMPicture);
}

void CGPGraphics::Render(void)
{
	if (!m_pComms->CheckForControlMMF()) return;
	
	PIC_DrawPicture();
	PIC_IM_DrawPicture();
	ML_DrawMultiLineIngame();
	SL_DrawTextLineIngame();
	SI_UpdateScreenInfo();
	SI_DrawFPS();
	SS_MakeScreenshot();
}

void CGPGraphics::FontSetup(void)
{
    memset(&m_LogFont,0,sizeof(LOGFONT));
    
    m_LogFont.Width				= 0; 
    m_LogFont.MipLevels			= 1; 
    m_LogFont.Italic			= FALSE; 
    m_LogFont.CharSet			= ANSI_CHARSET; 
    m_LogFont.OutputPrecision	= OUT_TT_PRECIS; 
    m_LogFont.Quality			= DEFAULT_QUALITY; 
    swprintf_s(m_LogFont.FaceName,32,L""); 
}

void CGPGraphics::SL_DrawTextLineIngame(void)
{
    // First, check if there is anything to show
	bool bShowMe[GP_NUMBER_OF_SINGLE_LINE_FIELDS];
	bool bSomethingToShow = false;
	
	for (BYTE i = 0; i<GP_NUMBER_OF_SINGLE_LINE_FIELDS; i++)
	{
        bShowMe[i] = m_pComms->MMF_SL_GetShowText(i);	
		if (bShowMe[i]) bSomethingToShow = true;
	}
	if (!bSomethingToShow) return;
	    
    // Now, step through the different textfields
	for (BYTE i = 0; i<GP_NUMBER_OF_SINGLE_LINE_FIELDS; i++)
	{
		if (bShowMe[i])
		{
			// Get all the text data
			WORD      wXPos, wYPos;
			LPCWSTR   pText;
			D3DCOLOR  dwTextColor;
			bool      bBlackBackground, bTextBold;
			BYTE      cSize, cFontFamily;
        
			if (m_pComms->MMF_SL_GetTextLineData(i, wXPos, wYPos, pText, dwTextColor, bBlackBackground, cSize, bTextBold, cFontFamily))
			{ 
				m_LogFont.Height = cSize; 
		        m_LogFont.Weight = (bTextBold?FW_BOLD:FW_NORMAL);
				m_LogFont.PitchAndFamily = DEFAULT_PITCH|cFontFamily;
	
				LPD3DXFONT pFont = NULL;
				if (D3DXCreateFontIndirect(m_pDevice,&m_LogFont,&pFont) == D3D_OK)
				{
                    RECT textPos = {0,0,0,0};

					if (m_pDevice->BeginScene()==D3D_OK)
					{
						pFont->DrawText(NULL,pText,-1,&textPos,DT_CALCRECT|DT_TOP|DT_SINGLELINE|DT_LEFT|DT_NOCLIP,dwTextColor);
        
						textPos.left   += wXPos;
						textPos.right  += wXPos;
						textPos.top    += wYPos;
						textPos.bottom += wYPos;

						if (bBlackBackground) m_pDevice->Clear(1,(D3DRECT*)&textPos,D3DCLEAR_TARGET,0x000000,0.0f,0);
                        pFont->DrawText(NULL,pText,-1,&textPos,DT_TOP|DT_SINGLELINE|DT_LEFT|DT_NOCLIP,dwTextColor);
         
						m_pDevice->EndScene();
					}
                    SAFERELEASE(pFont);
				} 
			} 
		} 
	} 
}

void CGPGraphics::ML_DrawMultiLineIngame(void)
{
    // First, check if there is anything to show
	bool bShowMe[GP_NUMBER_OF_MULTI_LINES_FIELDS];
	bool bSomethingToShow = false;
	
	for (BYTE i = 0; i<GP_NUMBER_OF_MULTI_LINES_FIELDS; i++)
	{
        bShowMe[i] = m_pComms->MMF_ML_GetShowText(i);	
		if (bShowMe[i]) bSomethingToShow = true;
	}
	if (!bSomethingToShow) return;

	// Now, step through the different textfields
	for (BYTE i = 0; i<GP_NUMBER_OF_MULTI_LINES_FIELDS; i++)
	{
		if (bShowMe[i])
		{
            WORD      wPosX, wPosY, wSizeX, wSizeY;
			LPCWSTR   pText;
			D3DCOLOR  dwTextColor;
			bool      bBlackBackground;
			BYTE      cFontSize, cFontFamily;
			bool	  bTextBold;
        
			if (m_pComms->MMF_ML_GetTextMultilineData(i, wPosX, wPosY, pText, dwTextColor, bBlackBackground, cFontSize, bTextBold, wSizeX, wSizeY, cFontFamily))
			{
				m_LogFont.Height = cFontSize; 
				m_LogFont.Weight = (bTextBold?FW_BOLD:FW_NORMAL);
				m_LogFont.PitchAndFamily = DEFAULT_PITCH|cFontFamily;
				LPD3DXFONT pFont = NULL;
				if (D3DXCreateFontIndirect(m_pDevice,&m_LogFont,&pFont) == D3D_OK)
				{
                    RECT textPos = {wPosX, wPosY, wPosX+wSizeX, wPosY+wSizeY};
				    if (m_pDevice->BeginScene()==D3D_OK)
					{
						if (bBlackBackground) m_pDevice->Clear(1,(D3DRECT*)&textPos,D3DCLEAR_TARGET,0x000000,0.0f,0);
				        pFont->DrawText(NULL,pText,-1,&textPos,DT_TOP|DT_LEFT|DT_WORDBREAK,dwTextColor);
       
						m_pDevice->EndScene();
					}
					SAFERELEASE(pFont);
				}
			}
		}
	}
}

void CGPGraphics::PIC_LoadPicture(void)
{
    // try to get path to picture
    LPCWSTR sPathToFile;
    if (!m_pComms->MMF_PIC_GetPathToFile(sPathToFile)) return;
    
    // Get rid of old pic if present
    SAFERELEASE(m_pPicture);
        
    // Check if file exists
    FILE *file;
	if (_wfopen_s( &file, sPathToFile, L"r" ) !=0 ) return;

    // it is there. continue
    fclose (file);
 
    // we get .bmp, .dds, .dib, .jpg, .png, and .tga
    D3DXCreateTextureFromFile(m_pDevice, sPathToFile, &m_pPicture);
}

void CGPGraphics::PIC_IM_LoadPicture(void)
{
	// Try to get raw data of picture
	BYTE* pPicData = NULL;
	UINT uiSize;
	if (!m_pComms->MMF_PIC_IM_GetPictureData(&pPicData, &uiSize)) return;

	// Get rid of old pic if present
	SAFERELEASE(m_pIMPicture);

	// Create texture from file in memory
	D3DXCreateTextureFromFileInMemory(m_pDevice, pPicData, uiSize, &m_pIMPicture);

	delete[] pPicData;
}

void CGPGraphics::PIC_DrawPicture(void)
{
    WORD wPosX, wPosY;
    bool bNew, bShow;
    
    if (!m_pComms->MMF_PIC_GetPictureNewShowPosition(wPosX, wPosY, bNew, bShow)) return; 

    if (bNew) PIC_LoadPicture(); 

    if (!m_pPicture) return;
    if (!bShow) return;
    
    // Now, it should be there. Draw it.
    LPD3DXSPRITE pSprite;
	
    HRESULT hRes = D3DXCreateSprite(m_pDevice, &pSprite);
    if (hRes != D3D_OK) return;

    D3DXVECTOR3 vec;
    vec.x = (float)wPosX; vec.y = (float)wPosY; vec.z = 0.0f;
    
    pSprite->Begin(0);
    pSprite->Draw(m_pPicture, NULL, NULL, &vec, 0xFFFFFFFF);
    pSprite->End();
    
    pSprite->Release();
}

void CGPGraphics::PIC_IM_DrawPicture(void)
{
	WORD wPosX, wPosY;
	bool bNew, bShow;

	if (!m_pComms->MMF_PIC_IM_GetPictureNewShowPosition(wPosX, wPosY, bNew, bShow)) return;

	if (bNew) PIC_IM_LoadPicture();

	if (!m_pIMPicture) { return; }
	if (!bShow) { return; }

	// Now, it should be there. Draw it.
	LPD3DXSPRITE pSprite;

	HRESULT hRes = D3DXCreateSprite(m_pDevice, &pSprite);
	if (hRes != D3D_OK) return;

	D3DXVECTOR3 vec;
	vec.x = (float)wPosX; vec.y = (float)wPosY; vec.z = 0.0f;

	pSprite->Begin(0);
	pSprite->Draw(m_pIMPicture, NULL, NULL, &vec, 0xFFFFFFFF);
	pSprite->End();

	pSprite->Release();
}

void CGPGraphics::SI_UpdateScreenInfo(void)
{
	//D3DDISPLAYMODE dMode;	
	//m_pDevice->GetDisplayMode(0, &dMode); for windowed, this would only return full screen size

	D3DVIEWPORT9 vPort;
	HRESULT hRet = m_pDevice->GetViewport(&vPort);

	if (hRet == D3D_OK) m_pComms->MMF_SI_SetScreenSizeInfo(vPort.Width, vPort.Height);
	else m_pComms->MMF_SI_SetScreenSizeInfo(0, 0);
}

void CGPGraphics::SI_DrawFPS(void)
{
    if (!m_pComms->MMF_SI_GetShowFPS()) return; // if no MMF or showing off
    
	static DWORD dwFrames = 0;
    static DWORD dwOldTic = 0;
	static float fFPS     = 0.0f;

	dwFrames++;
	DWORD dwNewTic = ::GetTickCount();
	
	if ((dwNewTic - dwOldTic) > 1000)
	{
		float fDummy = (dwNewTic - dwOldTic)/1000.0f;
		fFPS   = (dwFrames/fDummy);
		dwOldTic = dwNewTic;
		dwFrames = 0;
	}

	wchar_t cText [15];
	swprintf_s(cText,15,L"FPS: %.1f",fFPS);
	
    // Define fps font
    m_LogFont.PitchAndFamily = DEFAULT_PITCH;
	m_LogFont.Height = 14;

    m_LogFont.Weight = FW_NORMAL;
	
	// Now Create the Font. Must do locally to release it afterwards. Return on error
	LPD3DXFONT pFont = NULL;
	if (D3DXCreateFontIndirect(m_pDevice,&m_LogFont,&pFont) != D3D_OK) return;

	// Store/calculate text rectangle  
	RECT textPos = {0,0,0,0};

    if (m_pDevice->BeginScene()==D3D_OK)
    {
        // finally, draw text
        pFont->DrawText(NULL,cText,-1,&textPos,DT_TOP|DT_SINGLELINE|DT_LEFT|DT_NOCLIP,D3DCOLOR_ARGB(200,255,255,0));
         
        m_pDevice->EndScene();
    }
   
    // Unlock it. Will cause exception error when kept (e.g., when created in Constructor)
	SAFERELEASE(pFont);
}

void CGPGraphics::SS_MakeScreenshot(void) 
{
	//D3DXIFF_BMP = 0,
    //D3DXIFF_JPG = 1,
	
	if (!m_pComms->MMF_SS_GetMakeScreenshot()) return;

	IDirect3DSurface9* buf; 
    
	//D3DDISPLAYMODE dMode;
	//m_pDevice->GetDisplayMode(0, &dMode);

	D3DVIEWPORT9 vPort;
	if (m_pDevice->GetViewport(&vPort) != D3D_OK)
	{
		m_pComms->MMF_SS_ClearMakeScreenshot();
		return;
	}

	if (m_pDevice->CreateOffscreenPlainSurface(vPort.Width, vPort.Height, D3DFMT_A8R8G8B8, D3DPOOL_SCRATCH, &buf, NULL) != D3D_OK) //D3DFMT_A8R8G8B8
	{
		m_pComms->MMF_SS_ClearMakeScreenshot();
		return;
	}

    //if (m_pDevice->GetFrontBufferData(0,frontbuf) != D3D_OK)
	if (m_pDevice->GetBackBuffer((UINT)0, (UINT)0, D3DBACKBUFFER_TYPE_MONO, &buf) != D3D_OK)
    {
	  buf->Release(); 
      m_pComms->MMF_SS_ClearMakeScreenshot();
	  return;
    }

	LPCWSTR sPathToFile;
    if (!m_pComms->MMF_SS_GetPathToScreenshot(sPathToFile))
	{
	  buf->Release(); 
      m_pComms->MMF_SS_ClearMakeScreenshot();
	  return;
    }
    
	D3DXIMAGE_FILEFORMAT DestFormat = D3DXIFF_BMP;
	if (m_pComms->MMF_SS_GetScreenshotFormat()) DestFormat = D3DXIFF_JPG;
	
	
	D3DXSaveSurfaceToFile(sPathToFile, DestFormat, buf, NULL, NULL);
 
	buf->Release();
	m_pComms->MMF_SS_ClearMakeScreenshot();
}
